#import "GlassboxPlugin.h"
#import "GlassboxAgent.h"
#import "GlassboxAgentHidden.h"

NSString *const GB_CORDOVA_COOKIE = @"Cookie";

@interface GlassboxPlugin ()

@property NSError *agentError;

@property NSDictionary *auth;

@end

@implementation GlassboxPlugin

- (void)pluginInitialize {
	NSError *error;
	[GlassboxAgent startDom:&error];
	self.auth = nil;
	self.agentError = error;
}

- (void)domReport:(CDVInvokedUrlCommand *)command {
	NSArray *args = command.arguments;
	CDVPluginResult *pluginResult = nil;
	if ([args count] == 4) {
		NSString *url = [args objectAtIndex:0];
		NSDictionary *headers = [args objectAtIndex:1];
		NSObject *httpBody = [args objectAtIndex:2];
		NSString *dataType = [args objectAtIndex:3];
		headers = [self applyAuthentication:headers];
		[GlassboxAgent reportDomEventWithUrl:url headers:headers httpBody:httpBody];
		pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK];
	} else {
		pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:@"Wrong number of arguments for domReport action"];
	}

	[self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
}

- (void)setSessionCookie:(CDVInvokedUrlCommand *)command {
	NSArray *args = command.arguments;
	CDVPluginResult *pluginResult = nil;
	if ([args count] == 1) {
		NSString *cookie = [args objectAtIndex:0];
		self.auth = [NSDictionary dictionaryWithObject:cookie forKey:GB_CORDOVA_COOKIE];
		pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK];
	} else {
		pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:@"Wrong number of arguments for setSessionCookie action"];
	}

	[self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
}

- (void)onBridgeReady:(CDVInvokedUrlCommand *)command {
	NSArray *args = command.arguments;
	CDVPluginResult *pluginResult = nil;
	if ([args count] == 1) {
		NSString *cookie = [args objectAtIndex:0];
		[GlassboxAgent onDomBridgeReady:cookie];
		pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK];
	} else {
		pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:@"Wrong number of arguments for onBridgeReady action"];
	}

	[self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
}

- (void)startScreen:(CDVInvokedUrlCommand *)command {
	NSArray *args = command.arguments;
	CDVPluginResult *pluginResult = nil;
	if ([args count] == 1) {
		NSString *screenName = [args objectAtIndex:0];
		[GlassboxAgent startScreen:screenName];
		pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK];
	} else {
		pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:@"Wrong number of arguments for startScreen action"];
	}

	[self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
}


- (NSDictionary *)applyAuthentication:(NSDictionary *)reqHeaders {
	NSString *cookie = [reqHeaders objectForKey:GB_CORDOVA_COOKIE];
	NSString *athCookie = self.auth ? [self.auth objectForKey:GB_CORDOVA_COOKIE] : nil;
	if (!athCookie) {
		return reqHeaders;
	}

	NSString *newCookie = athCookie;
	if (cookie) {
		newCookie = [NSString stringWithFormat:@"%@;%@", cookie, newCookie];
	}

	NSMutableDictionary *newHeaders = reqHeaders ? [NSMutableDictionary dictionaryWithDictionary:reqHeaders] : [NSMutableDictionary dictionary];
	[newHeaders setObject:newCookie forKey:GB_CORDOVA_COOKIE];
	return newHeaders;
}

- (void) reportEvent:(CDVInvokedUrlCommand*)command {
	CDVPluginResult* pluginResult = nil;

	if ([command.arguments count] == 2) {
		NSString* event = [command.arguments objectAtIndex:0];
		NSDictionary* params = [command.arguments objectAtIndex:1];

		[GlassboxAgent reportEvent:event parameters:params];

		pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_OK];
	}
	else {
		pluginResult = [CDVPluginResult resultWithStatus:CDVCommandStatus_ERROR messageAsString:@"Wrong number of arguments for reportEvent action"];
	}

	[self.commandDelegate sendPluginResult:pluginResult callbackId:command.callbackId];
}

@end
