
var { Platform, NativeModules, findNodeHandle } = require('react-native');

var StoryboardBridge = NativeModules.Storyboard;

class Storyboard {

	// gets the current screen from navigation state
	static getActiveRouteName(navigationState) {
		if (!navigationState) {
			return null;
		}
		const route = navigationState.routes[navigationState.index];
		// dive into nested navigators
		if (route.routes) {
			return Storyboard.getActiveRouteName(route);
		}
		return route.routeName;
	}

	static trackNavigation(AppContainer) {
		try {
			if (AppContainer) {
				var origFunc = AppContainer.prototype._onNavigationStateChange;
				AppContainer.prototype._onNavigationStateChange = function (prevState, currentState, action) {
					try {
						const currentRouteName = Storyboard.getActiveRouteName(currentState);
						const previousRouteName = Storyboard.getActiveRouteName(prevState);
						if (previousRouteName !== currentRouteName) {
							Storyboard.startScreen(currentRouteName);
						}
					} catch(ex) {
						console.log("Glassbox: Failed configuring screen name:", ex);
					} finally {
						if(origFunc && this.props) origFunc.call(this, prevState, currentState, action);
					}	
				};
			}
		} catch (e) {
			console.log("Glassbox: Failed starting Navigation support:", e);
		}
	}

	static start(reportUrl, appId) {
		this.startWith({
			reportUrl,
			appId
		});
	}

	static startWith({ reportUrl, appId } = {}) {
		if (!reportUrl || !appId) {
			console.log("Storyboard: expecting reportUrl and appid");
			return;
		}
		StoryboardBridge.start(reportUrl, appId);
	}

	static stop() {
		StoryboardBridge.stop();
	}

	static reportEvent(eventName, eventParams) {
		if (!eventName) {
			console.log("Glassbox: custom event must have name");
			return;
		}

		StoryboardBridge.reportEvent(eventName, eventParams);
	}

	static setViewAsSensitive(viewRef, flags) {
		if (!viewRef) {
			console.log("Glassbox: Can't set a view as Sensitive. no component ref");
			return;
		}

		var viewNativeHandle = findNodeHandle(viewRef);
		if (!viewNativeHandle) {
			console.log("Glassbox: Could not locate native handle.");
			return;
		}

		var maskingFlags = flags || { maskOnAlpha: false, maskOnHidden: false, omitAnalytics: false, maskOnAllControllers: true, touchMode: StoryboardTouchMode.DISPLAY };

		StoryboardBridge.setViewAsSensitive(viewNativeHandle, maskingFlags);
	}

	static disableViewHiding(viewRef) {
		if (!viewRef) {
			console.log("Glassbox: Can't set a view as not Sensitive. no component ref");
			return;
		}

		var viewNativeHandle = findNodeHandle(viewRef);
		if (!viewNativeHandle) {
			console.log("Glassbox: Could not locate native handle.");
			return;
		}

		StoryboardBridge.disableViewHiding(viewNativeHandle);
	}

	static setViewTag(viewRef, tag) {
		if (!viewRef) {
			return;
		}

		var viewNativeHandle = findNodeHandle(viewRef);
		if (!viewNativeHandle) {
			console.log("Glassbox: Could not locate native handle.");
			return;
		}

		StoryboardBridge.setViewTag(viewNativeHandle, tag);
	}

	static startScreen(screenName) {
		if (!screenName) {
			return;
		}

		StoryboardBridge.startScreen(screenName);
	}

	static endScreen() {
		StoryboardBridge.endScreen();
	}

	static trackView(viewRef) {
		if (!viewRef) {
			console.log("Glassbox: Can't track this view. no component ref");
			return;
		}

		var viewNativeHandle = findNodeHandle(viewRef);
		if (!viewNativeHandle) {
			console.log("Glassbox: Could not locate native handle.");
			return;
		}

		StoryboardBridge.trackView(viewNativeHandle);
	}
}

var StoryboardTouchMode = {
	NONE: 'NONE',
	DISPLAY: 'DISPLAY',
	CENTER: 'CENTER'
};



module.exports = { Storyboard, StoryboardTouchMode }
